<?php declare(strict_types = 0);
/*
** Glaber
** Copyright (C) 2001-2024 Glaber
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**/

namespace Widgets\SvgMap\Actions;
require_once __DIR__ . '/../includes/WidgetFieldUpdater.php';

use APP, API, 
	CArrayHelper, CSeverityHelper,
	CControllerDashboardWidgetView,
	CControllerResponseData,
	CRoleHelper,WidgetFieldUpdater;

require_once APP::getRootDir().'/include/blocks.inc.php';

class WidgetView extends CControllerDashboardWidgetView {
	protected array $ids = [];
	protected $actions;
	protected $template_text;
	protected $macroses;
	
	protected $widgetid;
	protected $dashboardid;
	protected $widget_data;

	protected $hosts = [], $groups = [], $items = [], $triggers = [];

	protected function doAction(): void {
		$this->loadWidgetData();
		
		$iterator_svg = simplexml_load_string($this->loadSvgMap(), "SimpleXMLElement");
		
		
        $setup_mode = $this->widget_data->GetFieldValue('setup_mode');
		
		if ($setup_mode) 
			$this->processSetupSVG($iterator_svg);
		else {
			$this->loadActions();
			$this->loadObjectStates();
			$this->processViewSVG($iterator_svg, [], "");	
		}
		
		$this->setResponse(new CControllerResponseData([
			'name' => $this->getInput('name', $this->widget->getDefaultName()),
			'setup_mode' => $setup_mode,
			'dashboardid' => $this->dashboardid,
			'widgetid' => $this->widgetid,
			'svgmap' => $iterator_svg->asXML(),
		]));
	}

	protected function loadActions() {
		$actions_json = $this->fields_values['actions'];
		$this->actions = json_decode($actions_json, true);
	}

	protected function loadObjectStates() {

		$hostids = []; 
		$itemids = [];
		$groupids = [];
		$triggerids = [];
		
		foreach ($this->actions as $action_id => $action) {
			if (isset($action['bind_object_type'])){
				$object_type = $action['bind_object_type'];
				if ( 0 == $object_type)
					continue;
				switch ($object_type) {
					case 1:
						if (isset( $action['hostid']) )
							$hostids[$action['hostid']]=1;
						break;
					case 2:
						if (isset( $action['groupid']) )
							$groupids[$action['groupid']]=1;
						break;
					case 3:
						if (isset( $action['itemid']) )
							$itemids[$action['itemid']]=1;
						break;
					case 4:
						if (isset( $action['triggerid']) )
							$triggerids[$action['triggerid']]=1;
						break;
				}
			}
		}

		$this->fetchObjectsAndProblems($hostids, $groupids, $itemids, $triggerids);
	}

	protected function fetchObjectsAndProblems($hostids, $groupids, $itemids, $triggerids) {
		if (count($hostids) > 0 )
			$this->hosts = API::Host()->get(
				[ 	'output' => ['hostid','host','status', 'name', 'description'],
			  		'hostids' => array_keys($hostids),
			  		'preservekeys' => true
				]);
		
		if ( count($groupids) > 0 ) {
			$this->groups = API::HostGroup()->get(
					[ 	'output' => ['groupid','name'],
						'groupids' => array_keys($groupids),
						'preservekeys' => true
					]);
		}

		if (count($itemids) > 0 )
			$this->items = API::Item()->get(
			[ 	'output' => ['itemid', 'snmp_oid', 'hostid', 'name', 'key_', 'status', 'units', 'lastdata', 'lastvalue', 'prevvalue'],
				'itemids' => array_keys($itemids),
				'preservekeys' => true
			]);

		if (count($triggerids) > 0 )
			$this->triggers = API::Trigger()->get(
			[ 	'output' => ['triggerid', 'description', 'status', 'value', 'priority', 'lastchange', 'comments', 'error', 'state'],
				'triggerids' => array_keys($triggerids),
				'preservekeys' => true
			]);

		$add_hosts = [];

		if ( count($groupids) > 0) {
			$this->hosts +=
				 API::Host()->get([
				'output' => ['hostid','host','status', 'name', 'description'],
				'selectHostGroups' => ['groupid'],
				'groupids' => array_keys($groupids),
				'filter' => [
					'maintenance_status' => null
				],
				'monitored_hosts' => true,
				'preservekeys' => true
			]);
		}

		if (count($this->hosts) > 0) {
			$hosts_triggers = \CZabbixServer::getHostTriggerCounters(\CSessionHelper::getId(), array_keys($this->hosts));
			
			foreach ($hosts_triggers as $idx => $trigger_cnt) {
				$this->hosts[$trigger_cnt['hostid']] += $trigger_cnt;
			
				//hosts that present in the array due to host selection on the map will have no groups, skipping them
				if (!isset($this->hosts[$trigger_cnt['hostid']]['hostgroups']))
					continue;

				foreach ($this->hosts[$trigger_cnt['hostid']]['hostgroups'] as $idx => $group) {
					$groupid = $group['groupid'];

					if (!isset($this->groups[$groupid]))
						continue;
					
					if (!isset($this->groups[$groupid]['problems']))
					$this->groups[$groupid]['problems'] = [0,0,0,0,0,0];

					foreach ($this->hosts[$trigger_cnt['hostid']]['problems_by_severity'] as $severity => $count)
						$this->groups[$groupid]['problems'][$severity] +=$count;

				}
			}
		}
	}

	protected function processSetupSVG($sxi) {
		for($sxi->rewind(); $sxi->valid(); $sxi->next() ) {
			$attrs = $sxi->attributes();
			if (isset($attrs['id'])) {
			
				if (!isset($attrs['cursor']))
					$sxi->addAttribute('cursor','pointer');
				if (!isset($attrs['class']))
					$sxi->addAttribute('class','svgmap_highlight');
				if (!isset($attrs['onclick']))
					$sxi->addAttribute('onclick','PopUp("widget.svgmap.rule.edit.view",
													{elementid:"'.$attrs['id'].'",
													 dashboardid:"'.$this->dashboardid.'",
													 widgetid:"'.$this->widgetid.'"},
													{dialogue_class: "modal-popup-generic"});');
			}

			if ($sxi->hasChildren()) {
				$this->processSetupSVG($sxi->current());
			}
		}
	}
	
	protected function processViewSVG($element, $context, $stack) {
		$childNodes = $element->children();
		$this->processAttrRules($element, $context);

		$attrs = $element->attributes();

		if (isset($attrs['id'])) {
			$attr_id = strval($attrs['id']);
				
			if ( isset($this->actions[$attr_id]) ) {
		 		if (isset( $this->actions[$attr_id] )) {

		 			[$node_rules, $new_context, $macroses] = $this->processAction($this->actions[$attr_id]);
		 			$this->processAttrRules($element, $node_rules);

					if (count($childNodes) > 0) {
						foreach ($childNodes as $childNode) 
							$this->processViewSVG($childNode, $new_context, $stack);
					}
					
					return;
				}
			}  
		}			
				
		foreach ($childNodes as $childNode) 
				$this->processViewSVG($childNode, $context, $stack);

	}

	 protected function processAction($action) {
		
		$element_rules = [];
		$context_rules = [];
		$macroses = [];
		
		if (isset($action['bind_object_type']) && $action['bind_object_type'] >0 ) {
			switch ($action['bind_object_type']) {
				case 1: //host
					if (isset ($action['hostid'] )&& is_numeric($action['hostid']) &&  $action['hostid']> 0 )
						$macroses = $this->createHostMacroses($action['hostid']);
						
					break;
				case 2: //group
					if (isset ($action['groupid'] )&& is_numeric($action['groupid']) &&  $action['groupid']> 0 )
						$macroses = $this->createGroupMacroses($action['groupid']);
					break;
				case 3: //group
					if (isset ($action['itemid'] )&& is_numeric($action['itemid']) &&  $action['itemid']> 0 )
						$macroses = $this->createItemMacroses($action['itemid']);
					break;
				case 4: //group
					if (isset ($action['triggerid'] )&& is_numeric($action['triggerid']) &&  $action['triggerid']> 0 )
						$macroses = $this->createTriggerMacroses($action['triggerid']);
					break;
			}	
							
			if (isset($action['fill_max_severity']) && $action['fill_max_severity']>0 && isset($macroses['{MAXSEVERITY_COLOR}']) && '0' != $macroses['{MAXSEVERITY_COLOR}']) {
				
				$fill_rule =   [ 'attr_name' => 'fill', 'attr_value' => '#'.$macroses['{MAXSEVERITY_COLOR}'], 'attr_action' => 'replace_fill'];
				
				$element_rules[] = $fill_rule;
				$context_rules[] = $fill_rule;
			}

			if (isset($action['urls'])) {
				$element_rules[]= ['attr_name' => 'class', 'attr_action' => 'set', 'attr_value' => 'svgmap_highlight'];
				$element_rules[] =['attr_name' => 'cursor', 'attr_action' => 'set', 'attr_value' => 'pointer'];
				$url_items = [];
				
				foreach ($action['urls'] as $link) {
					if (isset($link['name']) && strlen($link['name'] > 0 && isset($link['url']) && strlen($link['url'] > 0)))
						$url_items['items'][] = ['label'=> $link['name'], 'url'=> $link['url']];
				}
				
				$element_rules[] =['attr_name' => 'data-menu-popup', 'attr_action' => 'set', 'attr_value' => 
										'{"type":"custom", "data":'.json_encode($url_items).'}'];
			}
		}
		
		return [$element_rules, $context_rules, $macroses];
	}

	protected function processAttrRules($xml_element, $rules) {
		$attributes = $xml_element->attributes();

		foreach ($rules as $idx => $rule) {
			$attr_name = $rule['attr_name'];

			switch ($rule['attr_action']) {
				case 'replace_fill':
					if (isset($attributes->$attr_name) && 
						strtolower($attributes->$attr_name) != '#000000' &&
						strtolower($attributes->$attr_name) != '#ffffff') {
					 		$attributes->$attr_name = (string)$rule['attr_value'];
					 		$xml_element->addChild('title','Some freaking test');
					 	}
					break;

				case 'replace':

					if (isset($attributes->$attr_name)) 
						$attributes->$attr_name = (string)$rule['attr_value'];
				
					break;
				case 'set':

					if (isset($attributes->$attr_name)) 
						$attributes->$attr_name = $rule['attr_value'];
					else  
						$attributes->addAttribute($attr_name, $rule['attr_value']);
			}
		}
	}
	
	protected function getMaxSeverity(array $problems) {
		for ($i = 5; $i>=0; $i--) {
			if ($problems[$i] > 0 ) 
				return $i;
		}

		return 0;
	}
	protected function getMaxSeverityColor(array $problems) {
		for ($i = 5; $i>=0; $i--) {
			if ($problems[$i] > 0 ) 
				return CSeverityHelper::getColor($i);
		}

		return 0;
	}


	protected function createHostMacroses($hostid){
		
		if (!isset($this->hosts[$hostid]))
			return [];
		
		foreach ($this->hosts[$hostid] as $prop => $value) 
			$macro[strtoupper($prop)] = $value;

		$macro['{PROBLEMS_TOTAL}'] = array_sum($this->hosts[$hostid]['problems_by_severity']);
		unset($macro['PROBLEMS_BY_SEVERITY']);
		
		$macro['{MAXSEVERITY}'] = $this->getMaxSeverity($this->hosts[$hostid]['problems_by_severity']);
		$macro['{MAXSEVERITY_COLOR}'] = $this->getMaxSeverityColor($this->hosts[$hostid]['problems_by_severity']);

		return $macro;
	}
	
	protected function createGroupMacroses($groupid){
		
		if (!isset($this->groups[$groupid]))
			return [];

		foreach ($this->groups[$groupid] as $prop => $value) 
			$macro[strtoupper($prop)] = $value;

		$macro['{MAXSEVERITY}'] = $this->getMaxSeverity($this->groups[$groupid]['problems']);
		$macro['{MAXSEVERITY_COLOR}'] = $this->getMaxSeverityColor($this->groups[$groupid]['problems']);
	
		return $macro;
	}
	
	protected function createItemMacroses($itemid){


		return [];
	}
	
	protected function createTriggerMacroses($triggerid){
	
		return [];
	}
 
	protected function loadWidgetData() {
		$raw_input = $this->getRawInput();

		$this->widgetid = $raw_input['widgetid'];
		$this->dashboardid = $raw_input['dashboardid'];
		
		unset($raw_input);

		try {
            $this->widget_data = new WidgetFieldUpdater($this->dashboardid, $this->widgetid);
        } catch (Exception $e) {
            $this->setResponse(new CControllerResponseFatal());
            return;
        } 
	}

	protected function loadSvgMap () {
		return  $this->fields_values['svgmap'];
	}
}
